/*
    This file is part of HelpViewer (http://www.roard.com/helpviewer)
    Copyright (C) 2003      Nicolas Roard (nicolas@roard.com)
                  2020-2021 Riccardo Mottola <rm@gnu.org>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the
    Free Software Foundation, Inc.  
    51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
*/

#ifndef __NOTE_CELL_H__
#define __NOTE_CELL_H__

#include <Foundation/Foundation.h>
#include <AppKit/AppKit.h>
#include "GNUstep.h"

@interface NoteCell : NSTextAttachmentCell
{
    NSImage* _image;
    NSMutableAttributedString* _note;
    NSSize noteSize;
    NSSize _size;
    NSColor* _color;
    CGFloat imageBorder;
    CGFloat leadingMargin;
    CGFloat trailingMargin;
}
- (id) initWithTextView: (NSTextView*) textview;
- (NSSize) cellSize;
- (void) setImage: (NSImage*) img;
- (void) setColor: (NSColor*) color;
- (void) setText: (NSMutableAttributedString*) text;
- (void) resize: (id) sender;
- (void) resizeWithTextView: (NSTextView*) textview;
- (void) drawWithFrame: (NSRect) cellFrame
    inView: (NSView*) controlView;
@end

#endif
